/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qabstractstate.h"
#include "qabstractstate_p.h"
#include "qstatemachine.h"
#include "qstatemachine_p.h"

QT_BEGIN_NAMESPACE

/*!
  \class QtAbstractState

  \brief The QtAbstractState class is the base class of states of a QtStateMachine.

  

  The QtAbstractState class is the abstract base class of states that are part
  of a QtStateMachine. It defines the interface that all state objects have in
  common. QtAbstractState is part of \l{The State Machine Framework}.

  The parentState() function returns the state's parent state.

  \section1 Subclassing

  The onEntry() function is called when the state is entered; reimplement this
  function to perform custom processing when the state is entered.

  The onExit() function is called when the state is exited; reimplement this
  function to perform custom processing when the state is exited.
*/

QtAbstractStatePrivate::QtAbstractStatePrivate()
{
}

QtAbstractStatePrivate *QtAbstractStatePrivate::get(QtAbstractState *q)
{
    return q->d_func();
}

const QtAbstractStatePrivate *QtAbstractStatePrivate::get(const QtAbstractState *q)
{
    return q->d_func();
}

QtStateMachine *QtAbstractStatePrivate::machine() const
{
    Q_Q(const QtAbstractState);
    QObject *par = q->parent();
    while (par != 0) {
        if (QtStateMachine *mach = qobject_cast<QtStateMachine*>(par))
            return mach;
        par = par->parent();
    }
    return 0;
}

void QtAbstractStatePrivate::callOnEntry()
{
    Q_Q(QtAbstractState);
    q->onEntry();
}

void QtAbstractStatePrivate::callOnExit()
{
    Q_Q(QtAbstractState);
    q->onExit();
}

/*!
  Constructs a new state with the given \a parent state.
*/
QtAbstractState::QtAbstractState(QtAbstractStateGroup *parent)
    : QObject(
#ifndef QT_STATEMACHINE_SOLUTION
        *new QtAbstractStatePrivate,
#endif
        parent)
#ifdef QT_STATEMACHINE_SOLUTION
    , d_ptr(new QtAbstractStatePrivate)
#endif
{
#ifdef QT_STATEMACHINE_SOLUTION
    d_ptr->q_ptr = this;
#endif
}

/*!
  \internal
*/
QtAbstractState::QtAbstractState(QtAbstractStatePrivate &dd, QtAbstractStateGroup *parent)
    : QObject(
#ifndef QT_STATEMACHINE_SOLUTION
        dd,
#endif
        parent)
#ifdef QT_STATEMACHINE_SOLUTION
    , d_ptr(&dd)
#endif
{
#ifdef QT_STATEMACHINE_SOLUTION
    d_ptr->q_ptr = this;
#endif
}

/*!
  Destroys this state.
*/
QtAbstractState::~QtAbstractState()
{
#ifdef QT_STATEMACHINE_SOLUTION
    delete d_ptr;
#endif
}

/*!
  Returns this state's parent state, or 0 if the state has no parent state.
*/
QtAbstractStateGroup *QtAbstractState::parentState() const
{
    return qobject_cast<QtAbstractStateGroup*>(parent());
}

/*!
  \fn QtAbstractState::onExit()

  This function is called when the state is exited.  Reimplement this function
  to perform custom processing when the state is exited.
*/

/*!
  \fn QtAbstractState::onEntry()

  This function is called when the state is entered. Reimplement this function
  to perform custom processing when the state is entered.
*/

QT_END_NAMESPACE
