/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qeventtransition.h"
#include "qeventtransition_p.h"
#include "qboundevent_p.h"
#include <qdebug.h>

QT_BEGIN_NAMESPACE

/*!
  \class QtEventTransition

  \brief The QtEventTransition class provides a QObject-specific transition for Qt events.

  

  A QtEventTransition object binds a QtAbstractTransition object to a particular
  QObject.  QtEventTransition is part of \l{The State Machine Framework}.
*/

QtEventTransitionPrivate::QtEventTransitionPrivate()
{
    object = 0;
    transition = 0;
    registered = false;
}

QtEventTransitionPrivate *QtEventTransitionPrivate::get(QtEventTransition *q)
{
    return q->d_func();
}

/*!
  Constructs a new QtEventTransition object that binds the given \a transition to
  the given \a object.
  The new QtEventTransition object takes ownership of \a transition.
*/
QtEventTransition::QtEventTransition(QObject *object, QtAbstractTransition *transition)
#ifdef QT_STATEMACHINE_SOLUTION
    : QtTransition(*new QtEventTransitionPrivate, QEvent::Type(QEvent::User-1))
#else
    : QtTransition(*new QtEventTransitionPrivate, QEvent::Bound)
#endif
{
    Q_D(QtEventTransition);
    d->object = object;
    d->transition = transition;
    if (transition)
        transition->setParent(this);
}

namespace {

class EventTransition : public QtAbstractTransition
{
public:
    EventTransition(QEvent::Type type)
        : QtAbstractTransition(type) {}
protected:
    void onTransition() {}
    bool testCondition(QEvent *) const { return true; }
};

} // namespace

/*!
  Constructs a new QtEventTransition object associated with events of the given
  \a type for the given \a object.
*/
QtEventTransition::QtEventTransition(QObject *object, QEvent::Type type)
#ifdef QT_STATEMACHINE_SOLUTION
    : QtTransition(*new QtEventTransitionPrivate, QEvent::Type(QEvent::User-1))
#else
    : QtTransition(*new QtEventTransitionPrivate, QEvent::Bound)
#endif
{
    Q_D(QtEventTransition);
    d->registered = false;
    d->object = object;
    d->transition = new EventTransition(type);
    d->transition->setParent(this);
}

/*!
  Destroys this QObject event transition.
*/
QtEventTransition::~QtEventTransition()
{
}

/*!
  \reimp
*/
bool QtEventTransition::testCondition(QEvent *event) const
{
    Q_D(const QtEventTransition);
    QtBoundEvent *oe = static_cast<QtBoundEvent*>(event);
    return (oe->object() == d->object)
        && QtAbstractTransitionPrivate::get(d->transition)->eventTypes.contains(oe->event()->type())
        && (QtAbstractTransitionPrivate::get(d->transition)->callTestCondition(oe->event()));
}

/*!
  \reimp
*/
void QtEventTransition::onTransition()
{
}

QT_END_NAMESPACE
