/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qhistorystate.h"
#include "qhistorystate_p.h"

QT_BEGIN_NAMESPACE

/*!
  \class QtHistoryState

  \brief The QtHistoryState class provides 'pause and resume' control flow in a state graph.

  

  A history state is a pseudo-state that represents the child state that the
  parent state was in the last time the parent state was exited. A transition
  with a history state as its target is in fact a transition to one of the
  other child states of the parent state. QtHistoryState is part of \l{The
  State Machine Framework}.

  Use QtAbstractStateGroup::addHistoryState() to construct a history state.
*/

QtHistoryStatePrivate::QtHistoryStatePrivate()
    : defaultState(0)
{
}

QtHistoryState *QtHistoryStatePrivate::create(QtAbstractStateGroup::HistoryType type,
                                            QtAbstractStateGroup *parent)
{
    return new QtHistoryState(type, parent);
}

QtHistoryStatePrivate *QtHistoryStatePrivate::get(QtHistoryState *q)
{
    return q->d_func();
}

const QtHistoryStatePrivate *QtHistoryStatePrivate::get(const QtHistoryState *q)
{
    return q->d_func();
}

/*!
  \internal

  Constructs a new history state of the given \a type, with the given \a
  parent state.
*/
QtHistoryState::QtHistoryState(QtAbstractStateGroup::HistoryType type, QtAbstractStateGroup *parent)
    : QtAbstractState(*new QtHistoryStatePrivate, parent)
{
    Q_D(QtHistoryState);
    d->historyType = type;
}

/*!
  Destroys this history state.
*/
QtHistoryState::~QtHistoryState()
{
}

/*!
  Returns this history state's default state.  The default state indicates the
  state to transition to if the parent state has never been entered before.
*/
QtAbstractState *QtHistoryState::defaultState() const
{
    Q_D(const QtHistoryState);
    return d->defaultState;
}

/*!
  Sets this history state's default state to be the given \a state.
*/
void QtHistoryState::setDefaultState(QtAbstractState *state)
{
    Q_D(QtHistoryState);
    d->defaultState = state;
}

/*!
  \reimp
*/
void QtHistoryState::onEntry()
{
}

/*!
  \reimp
*/
void QtHistoryState::onExit()
{
}

QT_END_NAMESPACE
