/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


/*!
    \class QtParallelAnimationGroup
    \brief The QtParallelAnimationGroup class provides a parallel group of animations.
    
    
    \preliminary

    The animations are all started at the same time, and run in parallel. The animation group
    finishes when the longest lasting animation has finished.
*/

#include "qparallelanimationgroup.h"
#include "qparallelanimationgroup_p.h"

QT_BEGIN_NAMESPACE

/*!
    Constructs a QtParallelAnimationGroup.
    \a parent is passed to QObject's constructor.
*/
QtParallelAnimationGroup::QtParallelAnimationGroup(QObject *parent)
    : QtAnimationGroup(*new QtParallelAnimationGroupPrivate, parent)
{
}

/*!
    \internal
*/
QtParallelAnimationGroup::QtParallelAnimationGroup(QtParallelAnimationGroupPrivate &dd,
                                                 QObject *parent)
    : QtAnimationGroup(dd, parent)
{
}

/*!
    Destroys the animation group. It will also destroy all its animations.
*/
QtParallelAnimationGroup::~QtParallelAnimationGroup()
{
}

/*!
    \reimp
*/
int QtParallelAnimationGroup::duration() const
{
    Q_D(const QtParallelAnimationGroup);
    int ret = 0;

    for (int i = 0; i < d->animations.size(); ++i) {
        QtAbstractAnimation *animation = d->animations.at(i);
        const int currentDuration = animation->totalDuration();
        if (currentDuration == -1)
            return -1; // Undetermined length

        ret = qMax(ret, currentDuration);
    }

    return ret;
}

/*!
    \reimp
*/
void QtParallelAnimationGroup::updateCurrentTime(int msecs)
{
    Q_D(QtParallelAnimationGroup);
    if (d->animations.isEmpty())
        return;

    foreach (QtAbstractAnimation *animation, d->animations)
        animation->setCurrentTime(msecs);
}

/*!
    \reimp
*/
void QtParallelAnimationGroup::updateState(QtAbstractAnimation::State oldState,
                                          QtAbstractAnimation::State newState)
{
    Q_D(QtParallelAnimationGroup);
    QtAnimationGroup::updateState(oldState, newState);

    switch (newState) {
    case NotRunning:
        foreach (QtAbstractAnimation *animation, d->animations)
            animation->stop();
        d->disconnectUncontrolledAnimations();
        break;
    case Paused:
        foreach (QtAbstractAnimation *animation, d->animations)
            animation->pause();
        break;
    case Running:
        d->connectUncontrolledAnimations();
        foreach (QtAbstractAnimation *animation, d->animations) {
            animation->stop();
            animation->setDirection(d->direction);
            animation->start();
        }
        break;
    }
}

void QtParallelAnimationGroupPrivate::_q_uncontrolledAnimationFinished()
{
    Q_Q(QtParallelAnimationGroup);

    QtAbstractAnimation *animation = qobject_cast<QtAbstractAnimation *>(q->sender());
    Q_ASSERT(animation);

    if (animation->duration() == -1 || animation->loopCount() < 0)
        --uncontrolledAnimations;

    if (uncontrolledAnimations > 0)
        return;

    int maxDuration = 0;
    foreach (QtAbstractAnimation *a, animations)
        maxDuration = qMax(maxDuration, a->totalDuration());

    if (currentTime >= maxDuration)
        q->stop();
}

void QtParallelAnimationGroupPrivate::disconnectUncontrolledAnimations()
{
    Q_Q(QtParallelAnimationGroup);

    foreach (QtAbstractAnimation *animation, animations)
        QObject::disconnect(animation, SIGNAL(finished()), q, SLOT(_q_uncontrolledAnimationFinished()));

    uncontrolledAnimations = 0;
}

void QtParallelAnimationGroupPrivate::connectUncontrolledAnimations()
{
    Q_Q(QtParallelAnimationGroup);

    foreach (QtAbstractAnimation *animation, animations) {
        if (animation->duration() == -1 || animation->loopCount() < 0) {
            QObject::connect(animation, SIGNAL(finished()), q, SLOT(_q_uncontrolledAnimationFinished()));
            ++uncontrolledAnimations;
        }
    }
}

/*!
    \reimp
*/
void QtParallelAnimationGroup::animationRemovedAt(int)
{
    Q_D(QtParallelAnimationGroup);
    const int dura = duration();
    if (dura == -1)
        return;

    //we need to update the current time
    setCurrentTime( dura * d->currentLoop + qMax(dura, d->currentTime));
}

/*!
    \reimp
*/
void QtParallelAnimationGroup::updateDirection(QtAbstractAnimation::Direction direction)
{
    //we need to update the direction of the current animation
    if (state() != NotRunning) {
        foreach(QtAbstractAnimation *anim, animations()) {
            anim->setDirection(direction);
        }
    }
}


QT_END_NAMESPACE

#include "moc_qparallelanimationgroup.cpp"

