/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qstateaction.h"
#include "qstateaction_p.h"
#include <QtCore/qvarlengtharray.h>
#include <QtCore/qmetaobject.h>

QT_BEGIN_NAMESPACE

QtStateActionPrivate::QtStateActionPrivate()
{
}

QtStateActionPrivate::~QtStateActionPrivate()
{
}

QtStateActionPrivate *QtStateActionPrivate::get(QtStateAction *q)
{
    return q->d_func();
}

void QtStateActionPrivate::callExecute()
{
    Q_Q(QtStateAction);
    q->execute();
}

/*!
  \class QtStateAction

  \brief The QtStateAction class is the base class of QtState actions.

  

  A state action is added to a state by calling QtState::addEntryAction() or
  QtState::addExitAction(). QtStateAction is part of \l{The State Machine
  Framework}.

  \section1 Subclassing

  Subclasses must reimplement the execute() function.
*/

/*!
  Constructs a new QtStateAction object.
*/
QtStateAction::QtStateAction()
    : QObject(
#ifndef QT_STATEMACHINE_SOLUTION
        *new QtStateActionPrivate
#endif
        )
#ifdef QT_STATEMACHINE_SOLUTION
    , d_ptr(new QtStateActionPrivate)
#endif
{
#ifdef QT_STATEMACHINE_SOLUTION
    d_ptr->q_ptr = this;
#endif
}

/*!
  \internal
*/
QtStateAction::QtStateAction(QtStateActionPrivate &dd)
    : QObject(
#ifndef QT_STATEMACHINE_SOLUTION
        dd
#endif
        )
#ifdef QT_STATEMACHINE_SOLUTION
    , d_ptr(&dd)
#endif
{
#ifdef QT_STATEMACHINE_SOLUTION
    d_ptr->q_ptr = this;
#endif
}

/*!
  Destroys this QtStateAction object.
*/
QtStateAction::~QtStateAction()
{
#ifdef QT_STATEMACHINE_SOLUTION
    delete d_ptr;
#endif
}

/*!
  \fn QtStateAction::execute()

  Executes this action.
*/

QtStateSetPropertyActionPrivate *QtStateSetPropertyActionPrivate::get(QtStateSetPropertyAction *q)
{
    return q->d_func();
}

/*!
  \class QtStateSetPropertyAction

  \brief The QtStateSetPropertyAction class provides a set property action for QObjects.

  

  The QtStateSetPropertyAction class provides an action that sets a property of
  a QObject to a pre-defined value when a QtState is entered or exited.
  QtStateSetPropertyAction is part of \l{The State Machine Framework}.

  Typically you don't construct QtStateSetPropertyAction objects directly, but
  rather call the QtState::setPropertyOnEntry() function or the
  QtState::setPropertyOnExit() function.
*/

/*!
  Constructs a new QtStateSetPropertyAction object for the property named \a
  name of the given \a object, with the given \a value.
*/
QtStateSetPropertyAction::QtStateSetPropertyAction(
    QObject *object, const char *name, const QVariant &value)
    : QtStateAction(*new QtStateSetPropertyActionPrivate)
{
    Q_D(QtStateSetPropertyAction);
    d->object = object;
    d->name = name;
    d->value = value;
}

/*!
  Destroys this QtStateAbstractSetPropertyAction object.
*/
QtStateSetPropertyAction::~QtStateSetPropertyAction()
{
}

/*!
  Returns the value set by this action.
*/
QVariant QtStateSetPropertyAction::value() const
{
    Q_D(const QtStateSetPropertyAction);
    return d->value;
}

/*!
  Returns the object for which this action sets a property.
*/
QObject *QtStateSetPropertyAction::targetObject() const
{
    Q_D(const QtStateSetPropertyAction);
    return d->object;
}

/*!
  Returns the name of the property set by this action.
*/
QByteArray QtStateSetPropertyAction::propertyName() const
{
    Q_D(const QtStateSetPropertyAction);
    return d->name;
}

/*!
  \reimp
*/
void QtStateSetPropertyAction::execute()
{
    Q_D(QtStateSetPropertyAction);
    d->object->setProperty(d->name, d->value);
}

QtStateInvokeMethodActionPrivate *QtStateInvokeMethodActionPrivate::get(QtStateInvokeMethodAction *q)
{
    return q->d_func();
}

/*!
  \class QtStateInvokeMethodAction

  \brief The QtStateInvokeMethodAction class provides an invoke method action for QObjects.

  

  The QtStateInvokeMethodAction class provides an action that calls a method of
  a QObject when a QtState is entered or exited. QtStateInvokeMethodAction is
  part of \l{The State Machine Framework}.

  Typically you don't construct QtStateInvokeMethodAction objects directly, but
  rather call the QtState::invokeMethodOnEntry() function or the
  QtState::invokeMethodOnExit() function.
*/

/*!
  Constructs a new QtStateInvokeMethodAction object for the given \a method of
  the given \a object, with the given arguments, \a args.
*/
QtStateInvokeMethodAction::QtStateInvokeMethodAction(QObject *object, const char *method,
                                                   const QList<QVariant> &args)
    : QtStateAction(*new QtStateInvokeMethodActionPrivate)
{
    Q_D(QtStateInvokeMethodAction);
    d->object = object;
    d->methodIndex = -1;
    d->args = args;

    QVarLengthArray<char, 512> sig;
    int len = qstrlen(method);
    if (len <= 0)
        return;
    sig.append(method, len);
    sig.append('(');

    int paramCount;
    for (paramCount = 0; paramCount < args.size() && paramCount < 10; ++paramCount) {
        const char *tn = args.at(paramCount).typeName();
        len = qstrlen(tn);
        if (len <= 0)
            break;
        sig.append(tn, len);
        sig.append(',');
    }
    if (paramCount == 0)
        sig.append(')'); // no parameters
    else
        sig[sig.size() - 1] = ')';
    sig.append('\0');

    int idx = object->metaObject()->indexOfMethod(sig.constData());
    if (idx < 0) {
        QByteArray norm = QMetaObject::normalizedSignature(sig.constData());
        idx = object->metaObject()->indexOfMethod(norm.constData());
        if ((idx < 0) || (idx >= object->metaObject()->methodCount())) {
            qWarning("InvokeMethodAction: unable to find method '%s' of %s(%p)",
                     sig.constData(), object->metaObject()->className(), object);
            return;
        }
    }

    d->methodIndex = idx;
}

/*!
  Destroys this QtStateInvokeMethodAction object.
*/
QtStateInvokeMethodAction::~QtStateInvokeMethodAction()
{
}

/*!
  \reimp
*/
void QtStateInvokeMethodAction::execute()
{
    Q_D(QtStateInvokeMethodAction);
    if (!d->object || (d->methodIndex == -1))
        return;
    void *param[11];
    param[0] = 0; // return value
    for (int i = 0; i < 10; ++i)
        param[i+1] = (i < d->args.size()) ? const_cast<void*>(d->args.at(i).constData()) : (void*)0;
    (void)d->object->qt_metacall(QMetaObject::InvokeMetaMethod, d->methodIndex, param);
}

/*!
  Returns the object on which this action invokes a method.
*/
QObject *QtStateInvokeMethodAction::targetObject() const
{
    Q_D(const QtStateInvokeMethodAction);
    return d->object;
}

/*!
  Returns the signature of the method this action will invoke.
*/
QByteArray QtStateInvokeMethodAction::methodSignature() const
{
    Q_D(const QtStateInvokeMethodAction);
    if (!d->object || (d->methodIndex == -1))
        return QByteArray();

    return d->object->metaObject()->method(d->methodIndex).signature();
}

QT_END_NAMESPACE
