/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qstatefinishedtransition.h"
#include "qstatefinishedevent.h"
#include "qtransition_p.h"

QT_BEGIN_NAMESPACE

/*!
  \class QtStateFinishedTransition

  \brief The QtStateFinishedTransition class provides a transition that triggers when a state is finished.

  

  A state is finished when one of its final child states (a QtFinalState) is
  entered; this will cause a QtStateFinishedEvent to be generated. The
  QtStateFinishedTransition class provides a way of associating a transition
  with such an event. QtStateFinishedTransition is part of \l{The State Machine
  Framework}.

  \code
    QtStateMachine machine;
    QtState *s1 = new QtState(machine.rootState());
    QtState *s11 = new QtState(s1);
    QtFinalState *s12 = new QtFinalState(s1);
    s11->addTransition(s12);

    QtState *s2 = new QtState(machine.rootState());
    s1->addTransition(new QtStateFinishedTransition(s1), s2);
  \endcode

  \sa QtAbstractStateGroup::addFinishedTransition(), QtStateFinishedEvent
*/

class QtStateFinishedTransitionPrivate : public QtTransitionPrivate
{
    Q_DECLARE_PUBLIC(QtStateFinishedTransition)
public:
    QtStateFinishedTransitionPrivate();

    static QtStateFinishedTransitionPrivate *get(QtStateFinishedTransition *q);

    QtAbstractStateGroup *state;
};

QtStateFinishedTransitionPrivate::QtStateFinishedTransitionPrivate()
{
}

QtStateFinishedTransitionPrivate *QtStateFinishedTransitionPrivate::get(QtStateFinishedTransition *q)
{
    return q->d_func();
}

/*!
  Constructs a new QtStateFinishedTransition object associated with the given
  \a state.

  \sa QtAbstractStateGroup::addTransition(), QtAbstractStateGroup::addFinishedTransition()
*/
QtStateFinishedTransition::QtStateFinishedTransition(QtAbstractStateGroup *state)
    : QtTransition(*new QtStateFinishedTransitionPrivate,
#ifndef QT_STATEMACHINE_SOLUTION
                          QEvent::StateFinished
#else
                          QEvent::Type(QEvent::User-2)
#endif
        )
{
    Q_D(QtStateFinishedTransition);
    d->state = state;
}

/*!
  Destroys this QtStateFinishedTransition.
*/
QtStateFinishedTransition::~QtStateFinishedTransition()
{
}

/*!
  \reimp
*/
bool QtStateFinishedTransition::testCondition(QEvent *event) const
{
    Q_D(const QtStateFinishedTransition);
    QtStateFinishedEvent *sfe = static_cast<QtStateFinishedEvent*>(event);
    return (sfe->state() == d->state);
}

/*!
  \reimp
*/
void QtStateFinishedTransition::onTransition()
{
}

QT_END_NAMESPACE
