/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of a Qt Solutions component.
**
** GNU General Public License Usage
** This file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/


#include "qtransition.h"
#include "qtransition_p.h"
#include "qstateaction.h"
#include "qstateaction_p.h"

QT_BEGIN_NAMESPACE

/*!
  \class QtTransition

  \brief The QtTransition class provides an action-based transition.

  

  QtTransition provides an action-based transition; you add actions with the
  addAction() function. The transition executes the actions when the
  transition is triggered. QtTransition is part of \l{The State Machine
  Framework}.

  \code
  QtStateMachine machine;
  QtState *s1 = new QtState(machine.rootState());
  QtTransition *t1 = new QtTransition();
  QLabel label;
  t1->addAction(new QtTransition::SetPropertyAction(&label, "text", "Transition t1 was triggered"));
  QtState *s2 = new QtState(machine.rootState());
  s1->addTransition(t1, s2);
  \endcode

  Actions are executed in the order in which they were added.
*/

QtTransitionPrivate::QtTransitionPrivate()
{
}

QtTransitionPrivate::~QtTransitionPrivate()
{
    qDeleteAll(actions);
}

QtTransitionPrivate *QtTransitionPrivate::get(QtTransition *q)
{
    return q->d_func();
}

const QtTransitionPrivate *QtTransitionPrivate::get(const QtTransition *q)
{
    return q->d_func();
}

/*!
  Constructs a new transition associated with the given \a eventTypes.

  The transition should be added to the appropriate state using
  QtAbstractState::addTransition().
*/
QtTransition::QtTransition(const QList<QEvent::Type> &eventTypes)
    : QtAbstractTransition(*new QtTransitionPrivate, eventTypes)
{
}

/*!
  Constructs a new transition with the given \a eventType.

  You can add the transition to a state using
  QtAbstractState::addTransition().
*/
QtTransition::QtTransition(QEvent::Type eventType)
    : QtAbstractTransition(*new QtTransitionPrivate, eventType)
{
}

/*!
  \internal
*/
QtTransition::QtTransition(QtTransitionPrivate &dd, QEvent::Type eventType)
    : QtAbstractTransition(dd, eventType)
{
}

/*!
  \internal
*/
QtTransition::QtTransition(QtTransitionPrivate &dd, const QList<QEvent::Type> &eventTypes)
    : QtAbstractTransition(dd, eventTypes)
{
}

/*!
  Destroys this transition.
*/
QtTransition::~QtTransition()
{
}

/*!
  Instructs this QtTransition to set the property with the given \a name of the
  given \a object to the given \a value when the transition is taken.
*/
void QtTransition::setPropertyOnTransition(QObject *object, const char *name,
                                          const QVariant &value)
{
    addAction(new QtStateSetPropertyAction(object, name, value));
}

/*!
  Instructs this QtTransition to invoke the given \a method of the given \a
  object with the given \a arguments when the transition is taken.
*/
void QtTransition::invokeMethodOnTransition(QObject *object, const char *method,
                                           const QList<QVariant> &arguments)
{
    addAction(new QtStateInvokeMethodAction(object, method, arguments));
}

/*!
  Adds the given \a action to this transition.
  The action will be executed when the transition is triggered.
  The transition takes ownership of the action.
*/
void QtTransition::addAction(QtStateAction *action)
{
    Q_D(QtTransition);
    if (action)
        d->actions.append(action);
}

/*!
  \reimp
*/
void QtTransition::onTransition()
{
    Q_D(QtTransition);
    for (int i = 0; i < d->actions.size(); ++i)
        QtStateActionPrivate::get(d->actions.at(i))->callExecute();
}

QT_END_NAMESPACE
